<?php

namespace App\Listeners\Frontend;

use App\Models\Forums\ForumChannel;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Laravel\Cashier\Cashier;
use Laravel\Cashier\Events\WebhookHandled;
use Laravel\Cashier\Events\WebhookReceived;
use App\Models\Lab;
use App\Models\Lookups\Stripe_plans;
use phpDocumentor\GraphViz\Exception;


class StripeEventListener
{
    /**
     * Create the event listener.
     *
     * @return void
     */
//    public function __construct()
//    {
//        //
//    }

    /**
     * Handle the event.
     *
     * @param WebhookHandled $event
     * @return void
     */
    public function handle(WebhookHandled $event)
    {
        info('StripeEventListener/handle: ' . $event->payload['data']['object']['customer'] . ':' . $event->payload['type']);
        $payload = $event->payload;
        // find the user
        if ($user = Cashier::findBillable($payload['data']['object']['customer'])) {
            // what type is it?
            switch ($event->payload['type']) {
                case 'customer.subscription.updated':
                    // check the user's status and do some interesting things with roles
                    // get the newly subscribed plan
                    $newplan = Stripe_plans::where(['stripe_plan' => $payload['data']['object']['plan']['id']])->first();
                    // do they have active subscriptions?
                    if ($user->current_active_subscriptions->count()) {
                        // update the plan
                        //$oldplan = Stripe_plans::where(['stripe_plan' => $user->current_active_subscriptions->last()->stripe_plan])->first();
                        // swap the subscription
//                // get the subscription
                        $subscription = $user->subscription($user->current_active_subscriptions->last()->name);
// get the subscription *item*. Dunno why this is like it is
                        $item = $subscription->findItemOrFail($user->current_active_subscriptions->last()->stripe_plan);
// swap the subscription
                        $item->swap($newplan->stripe_plan);
                        // update the subscription name
                        $subscription['name'] = $newplan->name;
                        $subscription->save();
                    } else {
                        // from https://miguelpiedrafita.com/snippets/stripeCheckout
                        $session = $payload['data']['object'];
                        DB::transaction(function () use ($session, $user, $newplan) {
                            $sub = $user->subscriptions()->create([
                                'name' => $newplan->name,
                                'stripe_id' => $session['id'],
                                'stripe_status' => 'active',
                                'stripe_plan' => $session['plan']['id'],
                                'quantity' => 1,
                                'ends_at' => null,
                            ]);
                        });


                    }
                    // Is the new plan a lab plan?
                    if ($newplan->selector == 'lab') {
                        // and does the user not already have one?
                        if (!$user->my_old_labs) {
                            try {
                                $lab = new Lab(['name' => 'My awesome lab', 'user_limit' => isset($newplan->lab_limit) ? $newplan->lab_limit : 5]);
                                $lab->save();
                                $lab->members()->sync(array($user->id => array('admin' => 'true')));
                                // make a lab forums
                                // announcements
                                $annoucements = new ForumChannel(['name' => 'Announcements', 'user_id' => $user->id, 'lab_id' => $lab->id, 'is_announcement_forum' => 1]);
                                $annoucements->save();
                                // general
                                $general = new ForumChannel(['name' => 'General', 'user_id' => $user->id, 'lab_id' => $lab->id]);
                                $general->save();
                            } catch (\Exception $e) {
                                info(strval($e));
                            }
                        } else {
                            // they already have an old lab but it should be read only
                            foreach ($user->my_old_labs as $old_lab) {
                                // make the old labs writable
                                if($old_lab) {
                                    $old_lab->read_only = null;
                                    $old_lab->save();
                                }
                            }
                        }
                    }

                    // restore access to any readonly boards
                    $userBoards = $user->boards->whereNull('lab_id');
                    foreach ($userBoards as $readonlyboard) {
                        $readonlyboard->readonly = null;
                        $readonlyboard->save();
                    }

                    // deal with any pending data
                    // we'll do this later
//                    foreach ($user->pending_data as $pending_data) {
//                        //rehydrate the data
//                        $data = \json_decode($pending_data->pending_data);
//                        // fix up the labs
//                        if ($user->my_lab) {
//                            //  dd(auth()->user()->my_lab);
//                            Lab::find($user->my_lab->id)->update(['name' => $data->lab_name]);
//                        } else {
//                            try {
//                                if ($data->lab_name) {
//                                    // make a new lab, make this person an admin
//                                    $lab = new Lab(['name' => $data->lab_name]);
//                                    $lab->save();
//                                    $lab->members()->sync(array($user->id => array('admin' => 'true')));
//                                }
//                            } catch (\Exception $e) {
//// dunno what happens here.
//                            }
//                        }
//                        // clean up the data
//                        $pending_data->delete();
//                    }

                    // update the user roles based on the active subscription
                    $roles = [$newplan->role];
                    if ($user->hasRole('administrator')) {
                        $roles[] = 'administrator';
                    }
                    if ($user->hasRole('lab member') || $user->is_in_lab) {
                        $roles[] = 'lab member';
                    }
                    $user->syncRoles($roles);

                    break;
                case 'customer.subscription.deleted':
                case 'customer.deleted':
                    // there's no active subscriptions, drop back to free level

                    foreach ($user->current_active_subscriptions as $active_subscription) {
                        $subscription = $user->subscription($active_subscription->name);
                        $subscription->syncStripeStatus();
                    }

                    //make any lab this user is admin of read-only
                    foreach ($user->labs as $lab) {
                        if ($lab->pivot->admin == 'true') {
                            $lab->read_only = "true";
                            $lab->save();
                        }
                    }
//                    if ($user->my_lab) {
//                        // make the old labs read only
//                        $user->my_lab->read_only = 'true';
//                        $user->my_lab->save();
//                    }

                    // make all but the first 3 boards read only

                    $userBoards = $user->boards->whereNull('lab_id')->sortByDesc('created_at');
                    if ($userBoards->count() > 3) {
                        $readonlyboards = $userBoards->splice(3);
                        foreach ($readonlyboards as $readonlyboard) {
                            $readonlyboard->readonly = 'true';
                            $readonlyboard->save();
                        }
                    }

                    $roles = ['user'];
                    if ($user->hasRole('administrator')) {
                        $roles[] = 'administrator';
                    }
                    if ($user->hasRole('lab member')) {
                        $roles[] = 'lab member';
                    }
                    $user->syncRoles($roles);
                    break;
//                case 'customer.deleted':
                    //$plan = Stripe_plans::where(['stripe_plan' => $user->subscriptions->first()->stripe_plan])->first();
//                    try {
//                        // cancel the subscription
//                        foreach ($user->current_active_subscriptions as $active_subscription) {
//                            $subscription = $user->subscription($active_subscription->name);
//                            $subscription->syncStripeStatus();
//                        }
//
//                        //make any lab this user is admin of read-only
//                        foreach ($user->labs as $lab) {
//                            if ($lab->pivot->admin == 'true') {
//                                $lab->read_only = "true";
//                                $lab->save();
//                            }
//                        }
//
//                        // make all but the first 3 boards read only
//                        $userBoards = $user->boards->whereNull('lab_id')->sortByDesc('created_at');
//                        if ($userBoards->count() > 3) {
//                            $readonlyboards = $userBoards->splice(3);
//                            foreach ($readonlyboards as $readonlyboard) {
//                                $readonlyboard->readonly = 'true';
//                                $readonlyboard->save();
//                            }
//                        }
//
//                        // drop back to free tier role...
//                        // remove role,
//                        // add base user role
//                        // but preserving the lab role if it's there
//                        //20200525 @TODO is this redundant?
//                        $roles = ['user'];
//                        if ($user->hasRole('administrator')) {
//                            $roles[] = 'administrator';
//                        }
//                        if ($user->hasRole('lab member')) {
//                            $roles[] = 'lab member';
//                        }
//                        $user->syncRoles($roles);
//                    } catch (\Error $e) {
//
//
//                    }
                    break;
                case 'invoice.payment_action_required':
// handle some payment required action here
                    break;
                default:
                    break;

            }
        }
        return;
    }

    /**
     * Register the listeners for the subscriber.
     *
     * @param \Illuminate\Events\Dispatcher $events
     */
    public function subscribe($events)
    {
        // logger('StripeEventListener/subscribe: ');
        $events->listen(
            WebhookHandled::class,
            'App\Listeners\Frontend\StripeEventListener@handle'
        );

    }
}
