<?php

namespace App\Models\Boards;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class BoardFolder extends Model
{

    use SoftDeletes;

    protected $guarded = [];

    // explicitly define tables
    protected $table = 'board_folders';

    protected $fillable = [
        'user_id', 'name', 'label','lab_id','parent_id'
    ];

    // the boards in this folder
    public function boards()
    {
        return $this->hasMany('App\Models\Boards\Board', 'board_folder_id', 'id');
    }

    // folder owner
    public function user()
    {
        return $this->hasOne('App\Models\Auth\User', 'id', 'user_id');
    }

    // scope for user
    public function scopeForUser($query, $id){
        return $query-> where('user_id', '=', $id);
    }

    public function subFolders(){
        return $this->hasMany(BoardFolder::class,"parent_id")->orderBy('order');
    }
}
