var rulezH;
var rulezV;

const saveSuccessToastOptions = {
    showClass: {
        popup: 'animate__animated animate__backInDown'
    },
    hideClass: {
        popup: 'animate__animated animate__backInUp'
    },
    toast: true,
    position: 'top-end',
    showConfirmButton: false,
    timer: 2000,
    icon: "success",
    title: 'Saved',
    icon: 'success',
};

const saveFailToastOptions = {
    toast: true,
    position: 'top-end',
    showConfirmButton: false,
    timer: 3000,
    icon: "error",
    title: 'Could not save :(',
};

const initialiseCore = () => {
    // select2 initialisation
    $('.tagsselect').select2({
        placeholder: 'Tags...',
    });

    $('#colorWays').dropdown()

    // hide things when a
    $(window).on('shown.bs.modal', function () {
        $('.navbar-collapse').collapse('hide');
    });

    $('.chart-section').css("display", "none");

    // add tooltop functionality
    tippy('[data-tippy-content]');

    $('.spectrum-color-picker').spectrum({
        type: "color",
        showPaletteOnly: true,
        togglePaletteOnly: true,
        showInitial: true,
        hideAfterPaletteSelect: true,
    });

    // $("input[type='number']").inputSpinner();

    // adjust control to not be hidden
    $("#canvascontroltemplate").on('shown.bs.modal', (e) => {
        //console.log(canvas.getActiveObjects()[0].getScaledHeight())
        if (($("#canvascontroltemplate").offset().top + $("#canvascontroltemplate .modal-content").height()) > ($("#myCanvas").offset().top + $("#myCanvas").height())) {
            $("#canvascontroltemplate").css("top", $("#canvascontroltemplate").offset().top - $("#canvascontroltemplate .modal-content").height() - canvas.getActiveObjects()[0].getScaledHeight());
        }
    })

    /////////////////////////////////////////
    // Form submissions
    ///////////////////////////////////////
    // metadata submission form
    $('#metadata_update_form').on('submit', (e) => {
        // console.log('submitting metadata')
        e.preventDefault();
        updateMetadata()
    });

    // metadata size change
    $('input[name=size]').change(function () {
        if ($('input[name=size]:checked').val() == 'custom') {
            $('#custom_dimension_input').show()
        } else {
            $('#custom_dimension_input').hide()
        }

    });
    //////////////////////////////////////////////////////
    // FabricJS initialisation and supporting stuff
    /////////////////////////////////////////////////////

    fabric.Object.prototype.objectCaching = true;
    fabric.Object.NUM_FRACTION_DIGITS = 8

    //Initialise canvas
    canvas = new fabric.Canvas('myCanvas', {
        preserveObjectStacking: true,
        selection: true,
        noScaleCache: true,
        // renderOnAddRemove: false
    });
    // fabric.Object.prototype.transparentCorners = false;

    canvas.skipOffscreen = true;
    console.log('setting dimensions')
    canvas.setHeight(canvas_height);
    canvas.setWidth(canvas_width);
    canvas.setBackgroundColor("rgba(255,255,255,0.8)");
    canvas.altSelectionKey = 'ctrlKey';
    // initialise the ruler
    canvas_ruler = new ruler({
        container: document.querySelector('.canvas-container'),// reference to DOM element to apply rulers on
        rulerHeight: 15, // thickness of ruler
        fontFamily: 'arial',// font for points
        fontSize: '10px',
        strokeStyle: 'black',
        lineWidth: 1,
        smallTickDist: 1,
        medTickDist: 10,
        bigTickDist: 20,
        enableMouseTracking: true,
        //enableToolTip: true
    });
    canvas_ruler.api.setScale(0.265);
    //
    // Events
    //

    // this is fired when something is selected
    canvas.on('selection:created', (options) => {
        // console.log('selection:created', options);
        updateContextualControls(options.target)
    });

    // when something else is selected
    canvas.on('selection:updated', (options) => {
        // console.log('selection:updated');
        updateContextualControls(options.target)
        canvas.getObjects('polygon').forEach(function (o) {
            // polygonEditingOff(o)
        });
    });


    // when nothing is selected
    canvas.on('selection:cleared', (options) => {
        // $("#contextual_toolbar").collapse('hide');
        $('#selected_content_popover').hide();
        $("#text_toolbar").collapse('hide');
        $(".colorpicker-bs-popover").hide();
        canvas.getObjects('polygon').forEach(function (o) {
            // polygonEditingOff(o)
        });

        $(".top-buttons_right").css("display", "none");
        $(".top_open_menu").css("display", "none");
    });


    // capturing double-click events.
    canvas.on('mouse:dblclick', (options) => {
        //  console.log('Double clicked on:');
        //   console.log(options.target);
        if (options.target) {
            // if it's itext, start editing
            if (options.target.type == 'i-text') {
                $("#canvascontroltemplate").modal('hide')
                options.target.enterEditing();
            }
            //. if it's a chart, show the update function
            if (options.target.chartData) {
                //  console.log('is a chart: chartdata is')
                let chartdata = JSON.parse(options.target.chartData)
                editChart(chartdata, true, chartdata.type, options.target)
                //  openChartPopup(chartdata, true, chartdata.type, options.target)
                // let chartdata = options.target.chartData;
                // // console.log(chartdata);
                // openChartPopup(chartdata, true, chartdata.type, options.target)
            }
            // if it's a polygon, allow editing
            if (options.target.type == 'polygon') {
                // console.log('editing a polygon')
                // polygonEditing(options.target)
            }

        } else {
            $("#canvascontroltemplate").modal('hide')
            // console.log('Nothing was clicked on')
        }


    });

    //Prevent moving object out of the art board space
    canvas.on('object:moving', function (e) {
        let obj = e.target;
        // is
        if (!obj.isOnScreen()) {
            // make the mouse let go
            let evt = new MouseEvent("mouseup", {
                bubbles: true,
                cancelable: true,
                view: window
            });
            canvas.upperCanvasEl.dispatchEvent(evt);
            // is it up too high?
            if (obj.getBoundingRect().top < -obj.getBoundingRect().height) {
                // bounce it back
                let target = -(obj.getBoundingRect().height * 0.1);
                obj.animate('top', target, {
                    onChange: canvas.renderAll.bind(canvas),
                    duration: 500,
                })
            }
            // is it too low?
            if (obj.getBoundingRect().top > canvas.height) {
                // bounce it back
                let target = obj.canvas.height - (obj.getBoundingRect().height * 0.1) + obj.top - obj.getBoundingRect().top
                obj.animate('top', target, {
                    onChange: canvas.renderAll.bind(canvas),
                    duration: 500,
                })
            }

            // to the left?
            if (obj.getBoundingRect().left < -obj.getBoundingRect().width) {
                // bounce it back
                let target = -(obj.getBoundingRect().width * 0.1)
                obj.animate('left', target, {
                    onChange: canvas.renderAll.bind(canvas),
                    duration: 500,
                })
            }
            // to the right?
            if (obj.getBoundingRect().left > canvas.width) {
                // bounce it back
                let target = obj.canvas.width - (obj.getBoundingRect().width * 0.1) + obj.left - obj.getBoundingRect().left;
                obj.animate('left', target, {
                    onChange: canvas.renderAll.bind(canvas),
                    duration: 500,
                })
            }
        }


    });


    // Rotations
    const angles = [0, 45, 90, 135, 180, 225, 270, 360]
    canvas.on('object:rotating', function (e) {
        //console.log(e.e.shiftKey)
        if (e.e.shiftKey) {
            e.transform.target.snapAngle = 45;
            e.transform.target.snapThreshold = 44;
        } else if (e.e.altKey) {
            e.transform.target.snapAngle = 1;
        } else {
            e.transform.target.snapAngle = 22.5;
            e.transform.target.snapThreshold = 5;
        }
        canvas.getActiveObjects().forEach(function (o) {
            if (typeof o.getObjects === 'function') {
                o.getObjects().forEach(function (p) {
                    if (p.get('name') == 'scaleGroup') {
                        p.set('angle', (o.get('flipX') || o.get('flipY')) ? ((o.get('flipX') && o.get('flipY')) ? (0 - e.transform.target.angle) : (0 + e.transform.target.angle)) : 0 - e.transform.target.angle)
                    }
                })
            }
        })
    })

    // opacity slider
    $("#opacityRange").on('change', (e) => {
        //console.log(e)
        setSelectedObjectOpacity(e.target.value / 100);
        $("#opacity_number").text(e.target.value + '%');
    });


    $("#opacity_number").on('keyup', (e) => {
        //console.log(e)
        setSelectedObjectOpacity(e.target.value / 100);
        $("#opacityRange").val(e.target.value);
    });


    // copying from OS
    $(window).on('copy', copyFunction);
    // pasting an image from the clipboard
    $(window).on('paste', pasteFunction);

    // bind delete key
    $('html').keyup(function (e) {
        if (e.keyCode == 46) {
            if (!canvas.getActiveObject().isEditing) {
                deleteSelected();
            }
        }
    });

    //Bind Arrow keys for moving elements
    //@TODO this is weird, fix it!
    $('html').keydown(function (event) {
        //console.log('keydown', event.keyCode, event.which)
        const STEP = 1;
        let keyCode = event.keyCode || event.which;
        let activeGroup = canvas.getActiveObjects();
        // console.log("activeGroup",activeGroup);
        // console.log("canvas.getActiveObject()",canvas.getActiveObject());
        if (Array.isArray(activeGroup)) {
            activeGroup.forEach(obj => {
                switch (keyCode) {
                    case 37: // left
                        obj.set({originX: 'left'});
                        //   console.log(obj.left);
                        event.preventDefault();
                        if (obj.left > 0)
                            obj.left = obj.left - STEP;
                        break;
                    case 38: // up
                        event.preventDefault();
                        obj.set({originY: 'top'});
                        // console.log(obj.top);
                        if (obj.top > 0)
                            obj.top = obj.top - STEP;
                        break;
                    case 39: // right
                        obj.set({originX: 'center'});
                        //  console.log(obj.left);
                        event.preventDefault();
                        obj.left = obj.left + STEP;
                        break;
                    case 40: // down
                        event.preventDefault();
                        obj.set({originY: 'top'});
                        // console.log(obj.top);
                        obj.top = obj.top + STEP;
                        break;
                }
                obj.setCoords();
            });
            canvas.renderAll();
        }
    })


    // // select2 initialisation
    // $('.templateselect').select2({
    //     placeholder: 'Template type...',
    // });
    /////////////////////////////////////////
    // Zoom functionality
    //////////////////////////////////////////


    //////////////////////////////////////////////////
    // Dropping onto canvas
    ///////////////////////////////////////////////////
    $("#myCanvas").droppable({
        accept: ".clipart_preview_div, .user_image_preview_div, .polygon_preview_div, .userBackgroundImage",
        //  classes: {
        //      "ui-droppable-active": "ui-state-active",
        //      "ui-droppable-hover": "ui-state-hover"
        //  },
        drop: (event, ui) => {
            //console.log("ui...",ui);
            // is it an image?
            if (ui.draggable.hasClass('user_image_preview_div')) {
                addUserImageToCanvas($(ui.draggable[0]).data('id'), ui.offset.top - $("#myCanvas").offset().top, ui.offset.left - $("#myCanvas").offset().left)
            }
            // is it clipart?
            if (ui.draggable.hasClass('clipart_preview_div')) {
                addSVGClipart($(ui.draggable[0]).data('id'), ui.offset.top - $("#myCanvas").offset().top, ui.offset.left - $("#myCanvas").offset().left)
            }

            // is it a polygon?
            if (ui.draggable.hasClass('polygon_preview_div')) {
                addPolygon($(ui.draggable[0]).data('sides'), ui.offset.top - $("#myCanvas").offset().top, ui.offset.left - $("#myCanvas").offset().left)
            }
            //console.log();
            //console.log(event)
            //console.log("ui.offset.top...",ui.offset.top)
            if (ui.draggable.hasClass('userBackgroundImage')) {
                //console.log("user back ground id...",$(ui.draggable[0]).data('id'));
                updateBackgroundImage($(ui.draggable[0]).data('id'));
            }
//
        }
    });

    // //To group the objects
    // $('.group').click(groupObjects());
    //
    // //To ungroup the objects
    // $(".ungroup").click(unGroupObjects());

    // export filetype feedback
    $('input[name=export_filetype]').on('change', (e) => {
            handleDownloadFiletypeChange(e);
        }
    )
    // export resolution feedback
    $('input[name=export_resolution]').on('change', (e) => {
            handleDownloadResolutionChange(e);
        }
    )
    //  var arrow = new Arrow(canvas);

}

// restore the canvas state
const restoreState = () => {
// restore zoom state
    let zoom_factor;
    if ((typeof state !== 'undefined') && state.hasOwnProperty('zoom')) {
        zoom_factor = state.zoom;
    } else {
        if (canvas.width > canvas.height) {
            zoom_factor = ($(window).width() * 0.9) / canvas.width;
        } else {
            zoom_factor = ($(window).height() * 0.9) / canvas.height;
        }
    }
    canvas.setZoom(zoom_factor)
    zoomTo(zoom_factor);
    // restore ruler state
    if ((typeof state !== 'undefined') && state.hasOwnProperty('has_ruler')) {
        $(".canvas-layout").css('margin-top', (state.has_ruler ? '15px' : '5px'))
        canvas_ruler.api.toggleRulerVisibility(state.has_ruler);
    }
}

// group the objects
const groupObjects = () => {
    if (!canvas.getActiveObject()) {
        return;
    }
    if (canvas.getActiveObject().type !== 'activeSelection') {
        return;
    }
    // turn off the history briefly
    canvas.offHistory();
    // do the thing
    canvas.getActiveObject().toGroup();
    //turn history back on
    canvas.onHistory();
    canvas.getActiveObject().grouped = true;
    canvas.requestRenderAll();
    $('.group').hide();
    $('#ungroupObjectButton').show();
}

const tempGroupObjects = () => {
    if (!canvas.getActiveObject()) {
        return;
    }
    if (canvas.getActiveObject().type !== 'activeSelection') {
        return;
    }
    // turn off the history briefly
    canvas.offHistory();
    // do the thing
    canvas.getActiveObject().toGroup();
    //turn history back on
    canvas.onHistory();
    canvas.getActiveObject().grouped = true;
    canvas.requestRenderAll();
}

const unGroupObjects = () => {
    if (!canvas.getActiveObject()) {
        return;
    }
    if (canvas.getActiveObject().type !== 'group') {
        return;
    }
    canvas.getActiveObject().toActiveSelection();
    canvas.getActiveObject().grouped = false;
    canvas.requestRenderAll();
    $('.ungroup').hide();
    $('#groupObjectButton').show();
}

//Set the opacity of a selected object
const setSelectedObjectOpacity = (value) => {
    //console.log('setting opacity to:' + value)

    canvas.getActiveObjects().forEach((o) => {
        //    console.log('getActiveObjects o is:' + o)
        // get scale opacity
        let has_scale = false;
        let scaleOpacity = 0;
        if (typeof o.getObjects === 'function') {
            o.getObjects().forEach((p) => {
                if (p.get('name') != 'scaleGroup') {
                    has_scale = true;
                    scaleOpacity = p.get('opacity')
                }
            })

            o.set('opacity', value)
            // reset scale opacity
            if (has_scale) {
                o.getObjects().forEach((p) => {
                    if (p.get('name') == 'scaleGroup') {
                        p.set('opacity', scaleOpacity)
                    }
                })
            }
        } else {
            o.set('opacity', value)
        }
    })
    canvas.requestRenderAll();
}

// toggle scale layer
const toggleScale = (visible = null) => {

    canvas.getActiveObjects().forEach(function (o) {
        if (typeof o.getObjects === 'function') {
            o.getObjects().forEach(function (p) {
                if (p.get('name') == 'scaleGroup') {
                    p.set('visible', visible ? visible : !p.get('visible'));
                }
            })
        }
    })
    canvas.requestRenderAll();
}

//flip the currently selected object in the specified axis
const flip = (x_or_y) => {
    // console.log('flip_x')
    canvas.getActiveObjects().forEach(function (o) {
        o.set(x_or_y == 'x' ? 'flipX' : 'flipY', !o.get(x_or_y == 'x' ? 'flipX' : 'flipY'));
        if (typeof o.getObjects === 'function') {
            o.getObjects().forEach(function (p) {
                if (p.get('name') == 'scaleGroup') {
                    let center = p.getCenterPoint();
                    p.set({
                        originX: 'center',
                        originY: 'center',
                        left: center.x,
                        top: center.y
                    });
                    p.set(x_or_y == 'x' ? 'flipX' : 'flipY', !p.get(x_or_y == 'x' ? 'flipX' : 'flipY'));
                    p.set('angle', (o.get('flipX') || o.get('flipY')) ? ((o.get('flipX') && o.get('flipY')) ? (0 - o.angle) : (0 + o.angle)) : 0 - o.angle)
                    //p.set('top', o.get('flipY') ? -((o.get('height') / 2) - (p.get('width') / 2)) : ((o.get('height') / 2) - (p.get('width') / 2)))
                    p.set('top', o.get('flipY') ? -((o.get('height') / 2) - (p.get('width') / 2)) : ((o.get('height') / 2) - (p.get('width') / 2)))
                    p.set('left', o.get('flipX') ? -((o.get('width') / 2) - (p.get('width') / 2)) : ((o.get('width') / 2) - (p.get('width') / 2)))
                }
            })
        }
    })
    canvas.requestRenderAll();
}

const deleteSelected = () => {
    canvas.getActiveObjects().forEach((obj) => {
        canvas.remove(obj)
    });
    canvas.discardActiveObject().renderAll()
}

//Zooming. @TODO limit the zoom to a sensible limit
const zoomIn = () => {
    zoom(1.1);
    //canvas.setZoom(canvas.getZoom() * 1.1 )
}

const zoomOut = () => {
    zoom(0.9);
    //canvas.setZoom(canvas.getZoom() / 1.1 )
}

const zoom = (ratio) => {

    let canvasContainer = document.getElementById("myCanvas");
    let middleX = canvasContainer.offsetWidth / 2;
    let middleY = 0;
    //console.log('current zoom is:' + canvas.getZoom())
    let currentZoom = canvas.getZoom();
    let newZoom = currentZoom + (ratio - 1);
    let height = canvas.getHeight();
    let width = canvas.getWidth();
    let zoomBy = newZoom / currentZoom;  // the amount to scale canvas

    canvas.setZoom(newZoom);

    //canvas.zoomToPoint(new fabric.Point(width / 2, height / 2), newZoom);
    canvas.setHeight(height * zoomBy);
    canvas.setWidth(width * zoomBy);

    let scale = 0.265 / canvas.getZoom();
    let smallTick = Math.pow(2, Math.floor(Math.log(scale * 2) / Math.log(2))) * 4;
    canvas_ruler.api.setTicks(smallTick, smallTick * 10, smallTick * 20)
    canvas_ruler.api.setScale(scale);
    // console.log('CLosest 2 is');
    // console.log(Math.pow(2,Math.floor(Math.log(scale*2)/Math.log(2)))*8)

    // let medTick = smallTick

    //$("#zoom_disp").html("(" + Math.round(canvas.getZoom() * 100) + "%)");
}

// zoom to a given zoom factor
const zoomTo = (zoom_factor) => {
    let height = canvas.getHeight();
    let width = canvas.getWidth();
    canvas.setZoom(zoom_factor)
    //console.log('zoomTo zoom factor is:'+canvas.getZoom())
    canvas.setHeight(height * zoom_factor);
    canvas.setWidth(width * zoom_factor);
    let scale = 0.265 / zoom_factor;
    let smallTick = Math.pow(2, Math.floor(Math.log(scale * 2) / Math.log(2))) * 4;
    canvas_ruler.api.setTicks(smallTick, smallTick * 10, smallTick * 20)
    canvas_ruler.api.setScale(scale);
    canvas_ruler.api.setScale(0.265 / canvas.getZoom());
    // $("#zoom_disp").html("(" + Math.round(canvas.getZoom() * 100) + "%)");
}
// layers
const bringToFront = () => {
    let currentElement = canvas.getActiveObject();
    canvas.bringToFront(currentElement).requestRenderAll();
}

const sendToBack = () => {
    let currentElement = canvas.getActiveObject();
    canvas.sendToBack(currentElement).requestRenderAll();
}

const bringForward = () => {
    let currentElement = canvas.getActiveObject();
    canvas.bringForward(currentElement).requestRenderAll();
}

const sendBackwards = () => {
    let currentElement = canvas.getActiveObject();
    canvas.sendBackwards(currentElement).requestRenderAll();
}

// copy from board
const copyFunction = (e) => {
    // copy a little bit of text to override anything else
    let success = true;
    if (window.clipboardData) { // Internet Explorer
        window.clipboardData.setData("Text", ' ');
    } else {
        navigator.clipboard.writeText("<empty clipboard>").then(function () {
            /* clipboard successfully set */
        }, function () {
            alert('Copy and pasting not supported in this browser')
        });
    }
    canvas.getActiveObject().clone((cloned) => {
        _clipboard = cloned;
        _clipboard.from_copy = true;
    });
}
// bonus: pasting an image
const pasteFunction = (e) => {
    // is this an image from the destop?
    console.log();
    let isImage = false;
    let isText = false;
    let items = e.originalEvent.clipboardData.items;
    e.preventDefault();
    e.stopPropagation();
    let i;
    //Loop through files
    for (i = 0; i < items.length; i++) {
        console.log('type is: ' + items[i].type)
        console.log(items[i])
        // is it an image?
        if (items[i].type.indexOf('image') > -1) {
            isImage = true;
            let file = items[i],
                type = items[i].type;
            let imageData = file.getAsFile();

            let URLobj = window.URL || window.webkitURL;
            let img = new Image();
            img.src = URLobj.createObjectURL(imageData);
            fabric.Image.fromURL(img.src, function (img) {
                canvas.add(img);
                if (img.getScaledWidth() > img.getScaledHeight()) {
                    //  console.log(img.getScaledWidth())
                    if (img.getScaledWidth() > canvas.width) {
                        //  console.log('scaling to width')
                        img.scaleToWidth(canvas.width - 10)
                    }
                } else {
                    // console.log(img.getScaledHeight())
                    if (img.getScaledHeight() > canvas.height) {
                        //   console.log('scaling to height')
                        img.scaleToHeight(canvas.height - 10)
                    }
                }
                canvas.renderAll();
            });
        }
        // is it text
        if (items[i].type.indexOf('text') > -1) {
            isText = true;
        }
    }
    if (isText) {
        if (((canvas.getActiveObject() !== undefined) && (canvas.getActiveObject() !== null)) && (canvas.getActiveObject().type == 'i-text' || canvas.getActiveObject().type == 'textbox')) {
            if (canvas.getActiveObject().isEditing) {
                canvas.getActiveObject().text = canvas.getActiveObject().text.substring(0, canvas.getActiveObject().selectionStart) + e.originalEvent.clipboardData.getData("text") + canvas.getActiveObject().text.substring(canvas.getActiveObject().selectionStart);
            } else {
                canvas.getActiveObject().text += e.originalEvent.clipboardData.getData("text");
            }

            canvas.renderAll();
        } else {
// make a new textbox and populate
            let textbox = new fabric.Textbox(e.originalEvent.clipboardData.getData("text"), {
                left: 100,
                top: 100,
                fontSize: 18.667,
                fontFamily: $("#update_text_font").val()
            });

            canvas.add(textbox);
            canvas.renderAll();
        }
    }

    // if it's not an image, there's something else on the clipboard
    if (!isImage && !isText && (_clipboard)) {
// clone again, so you can do multiple copies.
        _clipboard.clone(function (clonedObj) {
            canvas.discardActiveObject();
            clonedObj.set({
                left: clonedObj.left + 10,
                top: clonedObj.top + 10,
                evented: true,
            });
            if (clonedObj.type === 'activeSelection') {
                // active selection needs a reference to the canvas.
                clonedObj.canvas = canvas;
                clonedObj.forEachObject(function (obj) {
                    canvas.add(obj);
                });
                // this should solve the unselectability
                clonedObj.setCoords();
            } else {
                canvas.add(clonedObj);
            }
            _clipboard.top += 10;
            _clipboard.left += 10;
            canvas.setActiveObject(clonedObj);
            canvas.requestRenderAll();
        });
    }

}

//Clone the selected objects.
const cloneSelected = () => {
    canvas.renderOnAddRemove = false;
    // little optimisation: group
    let ismultiselect = false;
    let tempGroup;
    if (canvas.getActiveObject().type === 'activeSelection') {
        tempGroup = canvas.getActiveObject().toGroup();
        ismultiselect = true;
    } else {
        tempGroup = canvas.getActiveObject();
    }

    canvas.setActiveObject(tempGroup);
    tempGroup.clone((clonedObj) => {
        if (ismultiselect) {
            tempGroup.toActiveSelection();
        }
        canvas.discardActiveObject();
        //let count = 0;
        clonedObj.set({
            left: clonedObj.left + 10,
            top: clonedObj.top + 10,
            evented: true,
        });
        if (clonedObj.type === 'activeSelection') {
            // active selection needs a reference to the canvas.
            clonedObj.canvas = canvas;
            clonedObj.forEachObject(function (obj) {
                canvas.add(obj);
                //count++;
            });
            // this should solve the unselectability
            clonedObj.setCoords();
        } else {
            canvas.add(clonedObj);
        }
        canvas.renderOnAddRemove = true;
        canvas.requestRenderAll();
        canvas.setActiveObject(clonedObj);
        if (ismultiselect) {
            canvas.getActiveObject().toActiveSelection();
        }


        //console.log('count is:' + count);
    }, ['has_scale', 'is_scale', 'name', 'clipart_id', 'colourway', 'available_colourways', 'title', 'scaleGroup', 'grouped']);
}


// save the canvas to the database
function save() {
    // canvas.todataURL is a promise now.
    // Make a thumnail
    let thumb = canvas.toDataURL({
        format: 'png',
        multiplier: 0.2
    })
        // then after making the thumbnail, push the board data to the database/storage
        .then((response) => {
            let vars = [];
            vars.push({
                name: 'data',
                value: JSON.stringify(canvas.toJSON(['id', 'clipart_id', 'colourway_id', 'available_colourways', 'has_scale', 'chartData', 'name', 'title', 'grouped']))
            });
            vars.push({
                name: 'state',
                value: JSON.stringify({
                    'zoom': canvas.getZoom(),
                    'has_ruler': !($(".rul_wrapper").css('display') == 'none')
                })
            });
            vars.push({
                name: 'thumb', value: response
            });
            //  saving = true;
            wait(true);
            $.ajax(
                {
                    url: baseurl + '/board/' + canvas_id + '/savecanvas',
                    data: vars,
                    type: "post",

                }).done(function (data) {
                // positive feedback
                Swal.fire(saveSuccessToastOptions).then(() => {
                    wait(false);
                })
            }).fail(function (jqXHR, ajaxOptions, thrownError) {
                // failure feedback
                Swal.fire(saveFailToastOptions).then(() => {
                    wait(false);
                })
            });
        })
}

// update the metadata to the backend
// DEPRECATED
// function updateMetadata() {
//     let formData = $("#metadata_update_form").serialize();
//     //console.log(formData)
//     wait(true);
//     axios.post(baseurl + '/board/' + canvas_id + '/update',
//         formData
//     ).then(function (response) {
//         //   console.log(response);
//         $("#metadata_modal").modal('hide');
//         $("#metadata_update_form button[type='submit']").removeAttr('disabled')
//         if (response.data.status == 0) {
//             $("#board_name").html($("#metadata_update_form :input[name='name']").val())
//             $("#cm_width").html(Math.round(($("#metadata_update_form :input[name='cust_width']").val() / 37.795) * 100) / 100)
//             $("#cm_height").html(Math.round(($("#metadata_update_form :input[name='cust_height']").val() / 37.795) * 100) / 100)
//             let currentZoom = canvas.getZoom();
//             canvas.setHeight($("#metadata_update_form :input[name='cust_height']").val() * currentZoom);
//             canvas.setWidth($("#metadata_update_form :input[name='cust_width']").val() * currentZoom);
//             Swal.fire(saveSuccessToastOptions);
//             //$('#savingfeedback').toast('show')
//         } else {
//             Swal.fire(saveFailToastOptions).then(() => {
//                 wait(false);
//             })
//         }
//     }).catch(function (error) {
//         //   console.log(error);
//         Swal.fire(saveFailToastOptions)
//
//     })
//         .finally(function () {
//             // always executed
//             wait(false)
//         });
// }


//Updates the board metadata from teh board details.
function saveBoardDetails() {
// fudge the dimensions to get cm from pixels
    $("#cust_height_metadata_hidden").val(Math.round($("#cust_height_disp").val() * 3.7795))
    $("#cust_width_metadata_hidden").val(Math.round($("#cust_width_disp").val() * 3.7795))
    let vars = $("#updateForm").serialize();
    //console.log(vars);
    wait(true);
    axios({
        url: baseurl + '/board/' + canvas_id + '/update',
        method: 'post',
        data: vars

    }).then((response) => {
        //  console.log(response.data);
        wait(false);
        // if it's all good
        if (response.data.status == '0') {
            // update the board name
            $("#board_name_disp").html(response.data.name)
            // update the board dimensions
            let currentZoom = canvas.getZoom();
            canvas.setHeight(response.data.height* currentZoom);
            canvas.setWidth(response.data.width* currentZoom);
            canvas.renderAll();
            // update teh dimensions display
            $("#board_dimensions").html(Math.round(response.data.width / 3.7795) + 'x' + Math.round(response.data.height / 3.7795))
            // show the saving success feedback
            Swal.fire(saveSuccessToastOptions);
            // hide the form
            showMenus('board_mainmenu_container');
        } else {
            Swal.fire(saveFailToastOptions)
        }
        // // update the board name

    }).catch(function (error) {
        // console.log(error);
        wait(false);
        Swal.fire(saveFailToastOptions)
    });
}

// const updatePrintSize = () => {
//     $("#input_cm_width").html(Math.round(($("#metadata_update_form :input[name='cust_width']").val() / 37.795) * 100) / 100)
//     $("#input_cm_height").html(Math.round(($("#metadata_update_form :input[name='cust_height']").val() / 37.795) * 100) / 100)
// }

const wait = (on, message = '') => {
    // console.log(on)
    if (on) {
        $('.load-spinner').modal('show');
    } else {
        setTimeout(function () {
            $('.load-spinner').modal('hide');
        }, 500)

    }
}

const handleDownloadFiletypeChange = (e) => {
    console.log("handleDownloadFiletypeChange:" + e)
    // console.log($("#download_filetype").val())
    $('input[name=export_filetype]:checked')
    $("#export_filetype_detail").html($('input[name=export_filetype]:checked').data('detail'));
    if (($('input[name=export_filetype]:checked').val() == 'svg') || ($('input[name=export_filetype]:checked').val() == 'pdf')) {
        $(".export-resolution-display").hide()
    } else {
        $(".export-resolution-display").show()
    }
}

const handleDownloadResolutionChange = (e) => {
    console.log("handleDownloadResolutionChange:" + e)
    $("#export_resolution_detail").html($('input[name=export_resolution]:checked').data('detail'));
}

// crop output to contents
const cropOutputToContents = () => {
    let options = {};
    $("#export_cropped").html('');
    if ($("#export_crop").is(':checked')) {
        // get all objects as an array
        let objects = canvas.getObjects();
        // get the highest object
        let top = objects.reduce((prev, curr) => {
            //console.log(prev)
            return prev.getBoundingRect().top < curr.getBoundingRect().top ? prev : curr;
        })
        // get the lowest object
        let bottom = objects.reduce((prev, curr) => {
            return (prev.getBoundingRect().top + prev.getBoundingRect().height) > (curr.getBoundingRect().top + curr.getBoundingRect().height) ? prev : curr;
        })
// get the left-most object
        let left = objects.reduce((prev, curr) => {
            return prev.getBoundingRect().left < curr.getBoundingRect().left ? prev : curr;
        })
// get the right-most object
        let right = objects.reduce((prev, curr) => {
            return (prev.getBoundingRect().left + prev.getBoundingRect().width) > (curr.getBoundingRect().left + curr.getBoundingRect().width) ? prev : curr;
        })
        options = {
            format: "png",
            left: left.getBoundingRect().left - 5,
            top: top.getBoundingRect().top - 5,
            width: ((right.getBoundingRect().left + right.getBoundingRect().width) - left.getBoundingRect().left) + 10,
            height: ((bottom.getBoundingRect().top + bottom.getBoundingRect().height) - top.getBoundingRect().top) + 10
        }
        $("#export_cropped").html('Cropped to content');
    }

    canvas.toDataURL(options).then((response) => {
        process_preview_watermark(response, watermark_path);
        // watermark([response, watermark_path])
        //     .image(watermark.image.lowerLeft(0.5))
        //     .then(function (img) {
        //         $('#downloadPannel').find('img').attr('src', $(img).attr('src'));
        //     });
        //$('#downloadPannel').find('img').attr('src', response);
    })
    // let image = canvas.toDataURL(options);

}

const downloadArtboard = () => {
    // $("#downloaddialog").modal('hide');

    //If only one file is selected the based on the file type, the art board is converted to selected file type and downloaded.
    if ($('input[name=export_filetype]:checked').length == 1) {
        switch ($('input[name=export_filetype]:checked').val()) {
            case 'png':
                //downloadAsPNG($("#download_resolution").val());
                downloadAsPNG($('input[name=export_resolution]:checked').val(), $("#export_crop").is(':checked'))
                break;
            case 'jpg':
                downloadAsJPG($('input[name=export_resolution]:checked').val(), $("#export_crop").is(':checked'))
                break;
            case 'svg':
                downloadAsSVG($("#export_crop").is(':checked'));
                break;
            case 'pdf':
                downloadAsPDF($("#export_crop").is(':checked'));
                break;
            default:
                downloadAsPNG(96)
                break;
        }
    }
    // else {
    //When more than one file type is selected, the files are compressed in to a zip file and then downloaded.
    // if ($('input[name=file_type]:checked').length > 1) {
    //     let zip = new JSZip();
    //     let zipFilename = "board_images.zip";
    //     let count = 0;
    //     let links = [];
    //     $('input[name=file_type]:checked').each((index, el) => {
    //         switch ($(el).val()) {
    //             case 'png':
    //                 links.push({
    //                     file: canvas.toDataURL({format: 'png', multiplier: $("#pngResolution").val() / 96}),
    //                     type: 'png',
    //                 });
    //                 break;
    //             case 'jpg':
    //                 links.push({
    //                     file: canvas.toDataURL({
    //                         format: 'jpeg',
    //                         quality: 0.7,
    //                         multiplier: $("#jpgResolution").val() / 96
    //                     }),
    //                     type: 'jpg'
    //                 });
    //                 break;
    //             case 'svg':
    //                 links.push({
    //                     file: "data:image/svg+xml," + encodeURIComponent(canvas.toSVG()),
    //                     type: 'svg'
    //                 });
    //                 break;
    //             case 'pdf':
    //                 $.ajax({
    //                     url: baseurl + '/board/downloadaspdf',
    //                     method: 'POST',
    //                     data: {
    //                         img: canvas.toDataURL({format: 'png', multiplier: 3.125})
    //                     },
    //                     responseType: 'blob', // important,
    //                     async: false,
    //                     success: function (response) {
    //                         const url = window.URL.createObjectURL(new Blob([response.data]));
    //                         const link = document.createElement('a');
    //                         link.href = url;
    //                         links.push({
    //                             file: link.href,
    //                             type: 'pdf'
    //                         });
    //                     },
    //                     error: function (error) {
    //
    //                     }
    //                 })
    //             default:
    //                 links.push({
    //                     file: canvas.toDataURL({format: 'png', multiplier: 300 / 96}),
    //                     type: 'png'
    //                 });
    //                 break;
    //         }
    //     });
    //     links.forEach(function (url, i) {
    //         let filename = board_name + '.' + url.type;
    //         // loading a file and add it in a zip file
    //         JSZipUtils.getBinaryContent(url.file, function (err, data) {
    //             if (err) {
    //                 throw err; // or handle the error
    //             }
    //             zip.file(filename, data, {binary: true});
    //             count++;
    //             if (count == links.length) {
    //                 zip.generateAsync({type: 'blob'}).then(function (content) {
    //                     saveAs(content, zipFilename);
    //                 });
    //             }
    //         });
    //     });
    // } else {
    //     alert("please choose file type");
    // }
//}

}

const downloadAsPNG = (dpi = 300, cropped = false) => {
    let options = {format: 'png', multiplier: dpi / 96};
    if (cropped) {
        let objects = canvas.getObjects();
        // get the highest object
        let top = objects.reduce((prev, curr) => {
            //console.log(prev)
            return prev.getBoundingRect().top < curr.getBoundingRect().top ? prev : curr;
        })
        // get the lowest object
        let bottom = objects.reduce((prev, curr) => {
            return (prev.getBoundingRect().top + prev.getBoundingRect().height) > (curr.getBoundingRect().top + curr.getBoundingRect().height) ? prev : curr;
        })
// get the left-most object
        let left = objects.reduce((prev, curr) => {
            return prev.getBoundingRect().left < curr.getBoundingRect().left ? prev : curr;
        })
// get the right-most object
        let right = objects.reduce((prev, curr) => {
            return (prev.getBoundingRect().left + prev.getBoundingRect().width) > (curr.getBoundingRect().left + curr.getBoundingRect().width) ? prev : curr;
        })
        options = {
            format: "png",
            multiplier: dpi / 96,
            left: left.getBoundingRect().left - 5,
            top: top.getBoundingRect().top - 5,
            width: ((right.getBoundingRect().left + right.getBoundingRect().width) - left.getBoundingRect().left) + 10,
            height: ((bottom.getBoundingRect().top + bottom.getBoundingRect().height) - top.getBoundingRect().top) + 10
        }
    }

    let download = canvas.toDataURL(options).then((response) => {
        process_download_watermark(response, watermark_path, 'png')
    })

}

const downloadAsJPG = (dpi = 300, cropped = false) => {
    let options = {format: 'jpeg', quality: 0.7, multiplier: dpi / 96,};
    if (cropped) {
        let objects = canvas.getObjects();
        // get the highest object
        let top = objects.reduce((prev, curr) => {
            //console.log(prev)
            return prev.getBoundingRect().top < curr.getBoundingRect().top ? prev : curr;
        })
        // get the lowest object
        let bottom = objects.reduce((prev, curr) => {
            return (prev.getBoundingRect().top + prev.getBoundingRect().height) > (curr.getBoundingRect().top + curr.getBoundingRect().height) ? prev : curr;
        })
// get the left-most object
        let left = objects.reduce((prev, curr) => {
            return prev.getBoundingRect().left < curr.getBoundingRect().left ? prev : curr;
        })
// get the right-most object
        let right = objects.reduce((prev, curr) => {
            return (prev.getBoundingRect().left + prev.getBoundingRect().width) > (curr.getBoundingRect().left + curr.getBoundingRect().width) ? prev : curr;
        })
        options = {
            format: 'jpeg', quality: 0.7, multiplier: dpi / 96,
            left: left.getBoundingRect().left - 5,
            top: top.getBoundingRect().top - 5,
            width: ((right.getBoundingRect().left + right.getBoundingRect().width) - left.getBoundingRect().left) + 10,
            height: ((bottom.getBoundingRect().top + bottom.getBoundingRect().height) - top.getBoundingRect().top) + 10
        }
    }

    let download = canvas.toDataURL(options).then((response) => {
        process_download_watermark(response, watermark_path, 'jpg')
    })

}

// unused for now
const downloadAsSVG = (cropped = false) => {
    let options = {};
    if (cropped) {
        let objects = canvas.getObjects();
        // get the highest object
        let top = objects.reduce((prev, curr) => {
            //console.log(prev)
            return prev.getBoundingRect().top < curr.getBoundingRect().top ? prev : curr;
        })
        // get the lowest object
        let bottom = objects.reduce((prev, curr) => {
            return (prev.getBoundingRect().top + prev.getBoundingRect().height) > (curr.getBoundingRect().top + curr.getBoundingRect().height) ? prev : curr;
        })
// get the left-most object
        let left = objects.reduce((prev, curr) => {
            return prev.getBoundingRect().left < curr.getBoundingRect().left ? prev : curr;
        })
// get the right-most object
        let right = objects.reduce((prev, curr) => {
            return (prev.getBoundingRect().left + prev.getBoundingRect().width) > (curr.getBoundingRect().left + curr.getBoundingRect().width) ? prev : curr;
        })
        options = {
            x: left.getBoundingRect().left - 5,
            y: top.getBoundingRect().top - 5,
            width: (((right.getBoundingRect().left + right.getBoundingRect().width) - left.getBoundingRect().left) + 10) + 'mm',
            height: (((bottom.getBoundingRect().top + bottom.getBoundingRect().height) - top.getBoundingRect().top) + 10) + 'mm'
        }
    }

    let link = document.createElement("a");
    let dataURL = "data:image/svg+xml," + encodeURIComponent(canvas.toSVG(options));
    // console.log(blob)
    link.href = dataURL;
    //@TODO make the export name the same as the canvas
    link.download = board_name + ".svg";
    link.click();
    link.remove();
}
//window.jsPDF = window.jspdf.jsPDF;


const downloadAsPDF = (cropped) => {

    let options = {format: 'png', multiplier: 3.125};
    if (cropped) {
        let objects = canvas.getObjects();
        // get the highest object
        let top = objects.reduce((prev, curr) => {
            //console.log(prev)
            return prev.getBoundingRect().top < curr.getBoundingRect().top ? prev : curr;
        })
        // get the lowest object
        let bottom = objects.reduce((prev, curr) => {
            return (prev.getBoundingRect().top + prev.getBoundingRect().height) > (curr.getBoundingRect().top + curr.getBoundingRect().height) ? prev : curr;
        })
// get the left-most object
        let left = objects.reduce((prev, curr) => {
            return prev.getBoundingRect().left < curr.getBoundingRect().left ? prev : curr;
        })
// get the right-most object
        let right = objects.reduce((prev, curr) => {
            return (prev.getBoundingRect().left + prev.getBoundingRect().width) > (curr.getBoundingRect().left + curr.getBoundingRect().width) ? prev : curr;
        })
        options = {
            format: "png",
            multiplier: 3.125,
            left: left.getBoundingRect().left - 5,
            top: top.getBoundingRect().top - 5,
            width: ((right.getBoundingRect().left + right.getBoundingRect().width) - left.getBoundingRect().left) + 10,
            height: ((bottom.getBoundingRect().top + bottom.getBoundingRect().height) - top.getBoundingRect().top) + 10
        }
    }
    // $("#downloaddialog").modal('hide');
    wait(true);
    let img = canvas.toDataURL(options).then((response) => {
        process_download_pdf_watermark(response, watermark_path)
        // axios({
        //     url: baseurl + '/board/downloadaspdf',
        //     method: 'POST',
        //     data: {
        //         img: response
        //     },
        //     responseType: 'blob', // important
        // }).then((response) => {
        //     const url = window.URL.createObjectURL(new Blob([response.data]));
        //     const link = document.createElement('a');
        //     link.href = url;
        //     link.setAttribute('download', board_name + '.pdf'); //or any other extension
        //     document.body.appendChild(link);
        //     link.click();
        //     wait(false);
        // }).finally(() => {
        //     wait(false);
        // });
    })

}

const toggleRuler = () => {
    $(".canvas-layout").css('margin-top', ($(".rul_wrapper").css('display') == 'none') ? '15px' : '5px')
    canvas_ruler.api.toggleRulerVisibility(($(".rul_wrapper").css('display') == 'none'));
}

// panel control- hide all panels except for the one opening
// there's got to be a better way of doing this
// const showPanel = (panel) => {
//     //console.log('showing:'+panel)
//     if ($("#" + panel).hasClass('show')) {
//         $("#" + panel).collapse('hide');
//     } else if ($(".selectionpanel.collapse.show").length > 0) {
//         $(".selectionpanel.collapse.show:not(#" + panel).collapse('hide').one('hidden.bs.collapse', (e) => {
//             $("#" + panel).collapse('show');
//         });
//     } else {
//         $("#" + panel).collapse('show')
//     }
// }


/**
 * Traverse objects in groups (and subgroups)
 */
function traverseObjects(objects, attr, val, objectList) {
    for (let i in objects) {
        if (objects[i]['type'] == 'group') {
            traverseObjects(objects[i].getObjects(), attr, val, objectList);
        } else if (objects[i][attr] == val) {
            objectList.push(objects[i]);
        }
    }
}


const lockAll = () => {
    canvas.getObjects().forEach((item) => {
        item.selectable = false;
        item.evented = false
        //  item.hoverCursor= 'pointer';
    });
    canvas.discardActiveObject().renderAll();
}

const unlockAll = () => {
    let items = canvas.getObjects();
    if (!items) {
        return;
    }
    items.forEach((item) => {
        if (item.selectable == false) {
            item.selectable = true;
            item.hoverCursor = 'move';
            item.evented = true;
        }
    });
    canvas.discardActiveObject().renderAll();
}

/* Open and close menus */
const showMenus = (id) => {

    if (id) {
        console.log(id)
        let display = $("#" + id).css("display");
        $(".left_menu").css("opacity", "1");
        $(".board-tool-section").css('display', 'none');
        $(".board-top-tool-section").css("display", 'none');
        $(".board_mainmenu_container").css("display", 'none')
        if (display == "none") {
            $("#" + id).css("display", "block");
        } else {
            $("#" + id).css("display", "none");
        }
        //if open download menu means showing the current canvas as image
        //let image = canvas.toDataURL();
        if (id == "downloadPannel") {

            canvas.toDataURL("image/png").then((response) => {
                process_preview_watermark(response, watermark_path);
                //$('#downloadPannel').find('img').attr('src', response);
                //$('#downloadPannel').find('img').attr('src', response);
            });

        }
        if (id == "sharePannel") {
            canvas.toDataURL("image/png").then((response) => {
                console
                $('#sharePannel').find('img').attr('src', response);
            })

        }
        //if open edit board popup means add one active class for that div
        if (id == "board_mainmenu_container") {
            if (display == "none") {
                $('.board_name_input').addClass('active');
            } else {
                $('.board_name_input').removeClass('active');
            }
        }
        if (id == "groupObject") {
            showGroupObjectImage();
        }
    } else {
        console.log('clicked on nothing')
        $(".left_menu").css("opacity", "1");
        $(".board-tool-section").css('display', 'none');
        $(".board-top-tool-section").css("display", 'none');
        $(".board_mainmenu_container").css("display", 'none');
        $("#" + id).css("display", "none");

        $('.board_name_input').removeClass('active');

    }
}


//Decrease opacity of active menu
// $(".left_menu").click(function () {
//     if ($(this).attr('data-role') != "text") {
//         $(this).css('opacity', '0');
//     }
// })

//Clear all selections in download popup
function clearDownloadFilter() {
    $('input[name=file_type]').prop("checked", false);
}

//To share canvas via social media
function share(provider) {
    let image = canvas.toDataURL("image/png");

    $.ajax({
        url: baseurl + '/board/tempFileUpload',
        type: "POST",
        data: {img: image},
        success: function (response) {
            //  console.log(response);
            return false;
        },
        error: function (error) {
            //console.log("error...", error);
        }
    });
    // return false;
    // var alt = board_name;
    // switch(provider){
    //     case 'facebook':
    //         //window.open('http://www.facebook.com/sharer.php?u='+encodeURIComponent(image)+'&t='+encodeURIComponent(alt),'sharer','toolbar=0,status=0,width=626,height=436');
    //         FB.init({
    //             appId      : '1094943264225014',
    //             xfbml      : true,
    //             version    : 'v2.1'
    //         });
    //         FB.ui({
    //             method: 'share',
    //             href: image,
    //         }, function(response){});
    //         break;
    //     default:
    //         window.open('http://twitter.com/share?url='+encodeURIComponent(image)+'&text='+encodeURIComponent(alt), '', 'left=0,top=0,width=550,height=450,personalbar=0,toolbar=0,scrollbars=0,resizable=0');
    //         break;
    // }
    // return false;
}

//@TODO clean this up when save the copy of current art board

function saveDuplicate() {
    Swal.fire({
        showClass: {
            popup: 'animate__animated animate__fadeInDown'
        },
        hideClass: {
            popup: 'animate__animated animate__fadeOutUp'
        },
        title: 'Really duplicate board?',
        icon: 'info',
        confirmButtonText: 'OK',
        showCancelButton: true
    }).then((result) => {
        if (result.value) {
            let thumb = canvas.toDataURL({
                format: 'png',
                multiplier: 0.2
            }).then((response) => {
                let vars = [];

                vars.push({
                    name: 'data',
                    value: JSON.stringify(canvas.toJSON(['id', 'clipart_id', 'colourway_id', 'available_colourways', 'has_scale', 'chartData', 'name', 'title', 'grouped']))
                });
                vars.push({
                    name: 'state',
                    value: JSON.stringify({
                        'zoom': canvas.getZoom(),
                        'has_ruler': !($(".rul_wrapper").css('display') == 'none')
                    })
                });
                vars.push({
                    name: 'thumb', value: response
                });
                //  saving = true;
                wait(true);
                $.ajax(
                    {
                        url: baseurl + '/board/' + canvas_id + '/savecanvas',
                        data: vars,
                        type: "post",

                    }).done(function (data) {
                    // console.log('Save success!')
                    Swal.fire(saveSuccessToastOptions).then(() => {
                        // perform the duplication
                        $.ajax(
                            {
                                url: baseurl + '/board/' + canvas_id + '/saveDuplicate',
                                type: "POST",
                            }).done(function (data) {
                            // console.log('Save success!');
                            window.location.replace(baseurl + '/board/' + data.id);
                        }).fail(function (jqXHR, ajaxOptions, thrownError) {
                            // console.log('saving failed');
                            Swal.fire(saveFailToastOptions)
                        });
                    })
                    //$('#savingfeedback').toast('show')

                }).fail(function (jqXHR, ajaxOptions, thrownError) {
                    //  console.log('saving failed');
                    //  wait(false);
                    Swal.fire(saveFailToastOptions).then(() => {
                        wait(false);
                    })
                });
            });

        }
    })

}


//if open grouping object context menu, show the selected objects as image

function showGroupObjectImage() {
    let image = canvas.getActiveObject().toDataURL();
    $("#groupImagePreview").attr('src', image);
}


// undo/redo

const undo = () => {
    if (canvas.historyUndo.length > 1) {
        // is a group selected?
        // if (canvas.getActiveObject().grouped = true) {
        //
        // }
        canvas.undo()
    }
}

const redo = () => {
    canvas.redo()
}
