// Extended fabric line class
fabric.LineArrow = fabric.util.createClass(fabric.Line, {

    type: 'lineArrow',

    initialize: function (element, options) {
        options || (options = {});
        this.callSuper('initialize', element, options);
    },

    toObject: function () {
        return fabric.util.object.extend(this.callSuper('toObject'));
    },

    _render: function (ctx) {
        this.callSuper('_render', ctx);

        // do not render if width/height are zeros or object is not visible
        if (this.width === 0 || this.height === 0 || !this.visible) return;

        ctx.save();

        var xDiff = this.x2 - this.x1;
        var yDiff = this.y2 - this.y1;
        var angle = Math.atan2(yDiff, xDiff);
        ctx.translate((this.x2 - this.x1) / 2, (this.y2 - this.y1) / 2);
        ctx.rotate(angle);
        ctx.beginPath();
        //move 10px in front of line to start the arrow so it does not have the square line end showing in front (0,0)
        ctx.moveTo(10, 0);
        ctx.lineTo(-10, 15);
        ctx.lineTo(-10, -15);
        ctx.closePath();
        ctx.fillStyle = this.stroke;
        ctx.fill();

        ctx.restore();

    }
});

fabric.LineArrow.fromObject = function (object, callback) {
    callback && callback(new fabric.LineArrow([object.x1, object.y1, object.x2, object.y2], object));
};

fabric.LineArrow.async = true;


// the content template for the info popover for the board preview
const shapespopovercontent = '<div class="d-flex flex-column w-100" >' +
    '<div class="w-100 mb-1 ">' +
    '<div class="w-100 mb-1 info_details_description"></div>' +
    '</div>';

const border_selection_data = [
    {
        id: "",
        svg: '<svg style="display: inline; width: 4em;" viewBox="0 0 15 10" xmlns="http://www.w3.org/2000/svg">\n' +
            '  <line x1="0" y1="5" x2="15" y2="5" stroke="black" /></svg>',
        text: 'Solid'
    },
    {
        id: "2 2",
        svg: '<svg style="display: inline;  width: 4em;" viewBox="0 0 15 10" xmlns="http://www.w3.org/2000/svg">\n' +
            '  <line x1="0" y1="5" x2="15" y2="5" stroke="black" stroke-dasharray="2 2" />' +
            '</svg>',
        text: 'Little dashed'
    },
    {
        id: "4 1",
        svg: '<svg style="display: inline;  width: 4em;" viewBox="0 0 15 10" xmlns="http://www.w3.org/2000/svg">\n' +
            '  <line x1="0" y1="5" x2="15" y2="5" stroke="black" stroke-dasharray="4 1" />' +
            '</svg>',
        text: 'Long dashed'
    },

];


//shape fill color adding remove option in this template
// const fillColourPickerOptions = {
//     // strip the template down to the very basics
//     //@TODO extend this to have more options
//     template: '<div class="colorpicker">' +
//         '<div class="colorpicker-bar"><button class="btn btn-ghost-secondary btn-sm btn-block" onclick="fill_color_remove()">Remove</button></div>' +
//         '<div class="colorpicker-saturation"><i class="colorpicker-guide"></i></div>' +
//         '<div class="colorpicker-hue"><i class="colorpicker-guide"></i></div>' +
//         '<div class="colorpicker-alpha">' +
//         '   <div class="colorpicker-alpha-color"></div>' +
//         '   <i class="colorpicker-guide"></i>' +
//         '</div>',
//     fallbackColor: '#337AB7',
//     popover: {
//         placement: 'bottom',
//         animated:false
//     },
//
// }

// initialise teh shape functionality
const initShapes = () => {


    $("#update_shape_fill_colour").on('change.spectrum', function (e, tinycolor) {
        updateFillColour(tinycolor)
    });

    $("#update_shape_border_colour").on('change.spectrum', function (e, tinycolor) {
        updateBorderColour(tinycolor)
    });

    $('.polygon_preview_div').popover({
        html: true,
        animation: false,
        placement: 'right',
        trigger: 'hover',
        delay: {"show": 500, "hide": 100},
        template: '<div class="popover" role="tooltip"><h3 class="popover-header"></h3><div class="popover-body"></div></div>',
        content: shapespopovercontent,
        sanitize: false,
        boundary: 'window',
        offset: '0, 0',
        title: function () {
            return $(this).data('name')
        },
    }).on('shown.bs.popover', (e) => {
        console.log('showing popover')
        // fill in popover details
        $(".info_details_preview").attr('src', $(e.currentTarget).data('src'));
        $(".info_details_description").html($(e.currentTarget).data('description'));
    })

}

const updateFillColour = (colour) => {
    let currentElement = canvas.getActiveObject();
    if (currentElement) {
        if (colour) {
            currentElement.set('fill', colour.toString());
        } else {
            currentElement.set('fill', '');
        }
        canvas.renderAll();
    }
}

const updateBorderColour = (colour) => {
    console.log('updateBorderColour')
    let currentElement = canvas.getActiveObject();
    if (currentElement) {
        if (colour) {
            currentElement.set('stroke', colour.toString());
        } else {
            currentElement.set('stroke', '');
        }
        canvas.renderAll();
    }
    console.log(currentElement)
}

const updateStroke = (event = null) => {
    console.log('updateStroke')
    //   console.log('Log value...', $("input[name='shape_border_style']:checked").val().split('_'));
    // if (event) {
    //     event.stopPropagation();
    //     event.stopImmediatePropagation();
    // }
    let currentElement = canvas.getActiveObject();

    //currentElement.set('fill', $("#update_shape_fill_colour").colorpicker('getValue'));
    if (currentElement) {
        console.log('current element?');
        console.log(currentElement.getCoords());
        if ($("input[name='shape_border_style']:checked").val() == "none") {
            currentElement.set('stroke', null);
        } else {

            currentElement.set('stroke', $("#update_shape_border_colour").spectrum("get").toString());
        }

        currentElement.set({strokeWidth: parseInt($("#border_weight_range").val())});
        //  update_shape_border_weight
        // $("#border_weight_number").text($("#border_weight_range").val());
        currentElement.set("strokeDashArray", $("input[name='shape_border_style']:checked").val().split('_'));
        //currentElement.setCoords();
       canvas.renderAll();
        console.log(currentElement.getCoords());
    }

    canvas.getActiveObject().setCoords();
}

const getNewElementShapes = (page) => {


}

const shapes_dragoptions = {
    containment: "#myCanvas"
};


//////////////////////////////////////////////////////
// making a line
//////////////////////////////////////////////////////
// some placeholder variables
var lineIsDown = false
var currentLine;
var currentEnabledShape = null;

const enableLine = (e) => {
    // make the cursor into a cool crosshair
    canvas.defaultCursor = 'crosshair';
    canvas.selection = true;
    // add some events
    canvas.on('mouse:down', lineOnMouseDown);
    canvas.on('mouse:move', lineOnMouseMove);
    canvas.on('mouse:up', lineOnMouseUp);
    // toggle some visual confirmation that the line button is on
    $('.shape-button').removeClass('active')
    $(e).addClass('active')
    // show the shapes controls
    showShapesControls(true);
}

// start a fakies line to show where the real polyline will go
const lineOnMouseDown = (event) => {
    // lock canvas interaction
    lockAll();
    // flag that the line is actually down
    lineIsDown = true;
    // get the current mouse coordinats
    let pointer = canvas.getPointer(event.e);
    // save the origin for a minute
    textorigX = pointer.x;
    textorigY = pointer.y;
    // start the line
    let points = [pointer.x, pointer.y, pointer.x, pointer.y];
    currentLine = new fabric.Line(points, {
        stroke: 'grey',
        strokeWidth: 1,
        strokeDashArray: [5, 2]
    });
    canvas.add(currentLine).renderAll();
    currentLine.setCoords();
}

// make the end of the fakies linecfollow the mouse
const lineOnMouseMove = (event) => {
    if (!lineIsDown) return;
    let pointer = canvas.getPointer(event.e);
    if (event.e.shiftKey) {
        currentLine.set({
            x2: (Math.abs(pointer.x - textorigX) < Math.abs(pointer.y - textorigY)) ? textorigX : pointer.x,
            y2: (Math.abs(pointer.x - textorigX) > Math.abs(pointer.y - textorigY)) ? textorigY : pointer.y,
        });
    } else {
        currentLine.set({
            x2: pointer.x,
            y2: pointer.y
        });
    }


    canvas.renderAll();
}

// actually make the real line
const lineOnMouseUp = (event) => {
    // unlock the canvas for future work
    unlockAll();
    // clean up events
    canvas.off('mouse:down', lineOnMouseDown);
    canvas.off('mouse:move', lineOnMouseMove);
    canvas.off('mouse:up', lineOnMouseUp);
    // clean up cursor
    canvas.defaultCursor = 'auto';
    lineisDown = false;
    // get the final coordinates
    let pointer = canvas.getPointer(event.e);
    let p2 = {};
// make a polyline
    if (event.e.shiftKey) {
        p2 = {
            x: (Math.abs(pointer.x - textorigX) < Math.abs(pointer.y - textorigY)) ? textorigX : pointer.x,
            y: (Math.abs(pointer.x - textorigX) > Math.abs(pointer.y - textorigY)) ? textorigY : pointer.y
        }
    } else {
        p2 = {x: pointer.x, y: pointer.y}
    }
    let points = [{x: textorigX, y: textorigY}, p2];
    let polyline = new fabric.Polyline(points, {
        stroke: $("#update_shape_border_colour").spectrum("get").toString(),
        fill: '',
        strokeWidth: 1,
        transparentCorners: true,
        originY: 'top',
        strokeUniform: true,
    });
    // remove fakies guideline
    canvas.remove(currentLine).renderAll();
    // add the polyline
    canvas.add(polyline);
    canvas.renderAll();
    // enter editing mode- avoids a bug but also looks cool
    polyline.enterEditingMode();
    canvas.setActiveObject(polyline);
    // clean up the active look on the button
    $('.shape-button').removeClass('active')
}

//////////////////////////////////////////////////////
// making an arrow
//////////////////////////////////////////////////////
const enableArrow = (e) => {
    lockAll();
    canvas.defaultCursor = 'crosshair';
    canvas.selection = true;
    canvas.off('mouse:down', arrowOnMouseDown);
    canvas.off('mouse:move', arrowOnMouseMove);
    canvas.off('mouse:up', arrowOnMouseUp);
    canvas.on('mouse:down', arrowOnMouseDown);
    canvas.on('mouse:move', arrowOnMouseMove);
    canvas.on('mouse:up', arrowOnMouseUp);
    $('.shape-button').removeClass('active')
    $(e).addClass('active')
    showShapesControls(true);
}

// start a fakies line to show where the real polyline will go
const arrowOnMouseDown = (event) => {
    // lock canvas interaction

    // flag that the line is actually down
    lineIsDown = true;
    // get the current mouse coordinats
    let pointer = canvas.getPointer(event.e);
    // save the origin for a minute
    textorigX = pointer.x;
    textorigY = pointer.y;
    // start the line
    let points = [pointer.x, pointer.y, pointer.x, pointer.y];
    currentLine = new fabric.LineArrow(points, {
        stroke: 'grey',
        strokeWidth: 1,
        strokeDashArray: [5, 2]
    });
    canvas.add(currentLine).renderAll();
    currentLine.setCoords();
}

// make the end of the fakies linecfollow the mouse
const arrowOnMouseMove = (event) => {
    if (!lineIsDown) return;
    let pointer = canvas.getPointer(event.e);
    if (event.e.shiftKey) {
        currentLine.set({
            x2: (Math.abs(pointer.x - textorigX) < Math.abs(pointer.y - textorigY)) ? textorigX : pointer.x,
            y2: (Math.abs(pointer.x - textorigX) > Math.abs(pointer.y - textorigY)) ? textorigY : pointer.y,
        });
    } else {
        currentLine.set({
            x2: pointer.x,
            y2: pointer.y
        });
    }
    canvas.renderAll();
}

// actually make the real line
const arrowOnMouseUp = (event) => {
    // unlock the canvas for future work
    unlockAll();
    // clean up events
    canvas.off('mouse:down', arrowOnMouseDown);
    canvas.off('mouse:move', arrowOnMouseMove);
    canvas.off('mouse:up', arrowOnMouseUp);
    // clean up cursor
    canvas.defaultCursor = 'auto';
    lineisDown = false;
    // get the final coordinates
    let pointer = canvas.getPointer(event.e);
    let p2 = {};
// make a polyline
    if (event.e.shiftKey) {
        p2 = {
            x: (Math.abs(pointer.x - textorigX) < Math.abs(pointer.y - textorigY)) ? textorigX : pointer.x,
            y: (Math.abs(pointer.x - textorigX) > Math.abs(pointer.y - textorigY)) ? textorigY : pointer.y
        }
    } else {
        p2 = {x: pointer.x, y: pointer.y}
    }
    let points = [{x: textorigX, y: textorigY}, p2];

    let polyline = new fabric.Arrow(points, {
        stroke: $("#update_shape_border_colour").spectrum("get").toString(),
        fill: '',
        arrowSize: 10,
        strokeWidth: 1,
        transparentCorners: true,
        originY: 'top',
        strokeUniform: true,
    });
    // remove fakies guideline
    canvas.remove(currentLine).renderAll();
    // add the polyline
    canvas.add(polyline);
    canvas.renderAll();
    // enter editing mode- avoids a bug but also looks cool
    polyline.enterEditingMode();
    canvas.setActiveObject(polyline);
    // clean up the active look on the button
    $('.shape-button').removeClass('active')
}

//////////////////////////////////////////////////////
// making an polygon
//////////////////////////////////////////////////////
let currentPolyCount = 3;
let currentPolygon;


const enablePolygon = (e) => {
    lockAll();

    currentPolyCount = $(e).data('sides')
    canvas.defaultCursor = 'crosshair';
    canvas.selection = true;
    // work out the events
    canvas.off('mouse:down', polyOnMouseDown);
    canvas.off('mouse:move', polyOnMouseMove);
    canvas.off('mouse:up', polyOnMouseUp);
    canvas.on('mouse:down', polyOnMouseDown);
    canvas.on('mouse:move', polyOnMouseMove);
    canvas.on('mouse:up', polyOnMouseUp);
    $('.shape-button').removeClass('active')
    $(e).addClass('active')
    //showShapesControls(true);
}

// make a polygon
const polyOnMouseDown = (event) => {
    canvas.selection = false;
    // deselect everything
    canvas.discardActiveObject();
    lineIsDown = true;
    let pointer = canvas.getPointer(event.e, false);
    // start the line
    let points = PolyGen(currentPolyCount, 20, pointer.x, pointer.y)
    textorigX = pointer.x;
    textorigY = pointer.y;
    currentPolygon = new fabric.Polygon(points, {
        originX: "left",
        originY: "top",
        objectCaching: false,
        stroke: 'grey',
        strokeWidth: 1,
        fill: '',
        strokeDashArray: [5, 2]
        // left: pointer.x,
        // top:
    });
    canvas.add(currentPolygon)

    // polygon.exitEditingMode();
    // canvas.defaultCursor = 'default';
    //  polygon.enterEditingMode();
    // canvas.setActiveObject(currentPolygon);
    currentPolygon.setCoords();
    canvas.renderAll();
    //polygon.enterEditingMode();
}


// make the end of the fakies linecfollow the mouse
const polyOnMouseMove = (event) => {
    console.log('polyOnMouseMove')
    if (!lineIsDown) return;
    let pointer = canvas.getPointer(event.e);
    let hypot = Math.hypot(Math.abs(pointer.x - textorigX), Math.abs(pointer.y - textorigY))
    hypot = (hypot < 20) ? 20 : hypot;
    let points = PolyGen(currentPolyCount, hypot, pointer.x, pointer.y)
    currentPolygon.points = points;
    //canvas.setActiveObject(currentPolygon);
    currentPolygon.setCoords();
    canvas.renderAll();
}

const polyOnMouseUp = (event) => {
    // unlock the canvas for future work
    unlockAll();
    // clean up events
    canvas.off('mouse:down', polyOnMouseDown);
    canvas.off('mouse:move', polyOnMouseMove);
    canvas.off('mouse:up', polyOnMouseUp);
    // clean up cursor
    canvas.defaultCursor = 'auto';
    lineisDown = false;
    // get the final coordinates
    let pointer = canvas.getPointer(event.e);
// make a polyline
    let poly = new fabric.Polygon(currentPolygon.points, {
        originX: "center",
        originY: "center",
        objectCaching: false,
        stroke: 'black',
        strokeWidth: 1,
        fill: '',
        strokeUniform: true,
        // left: pointer.x,
        // top:
    });
    // remove fakies guideline
    canvas.remove(currentPolygon).renderAll();
    // add the polygon
    canvas.add(poly)
    canvas.renderAll();
    // enter editing mode- avoids a bug but also looks cool
    //currentPolygon.enterEditingMode();
    canvas.setActiveObject(poly);
    // clean up the active look on the button
    $('.shape-button').removeClass('active')
}
//////////////////////////////////////////////////////
// making an ellipse
//////////////////////////////////////////////////////
const enableEllipse = (e) => {

    lockAll();
    canvas.defaultCursor = 'crosshair';
    canvas.selection = true;
    canvas.off('mouse:down', ellipseOnMouseDown);
    canvas.off('mouse:move', ellipseOnMouseMove);
    canvas.off('mouse:up', ellipseOnMouseUp);
    canvas.on('mouse:down', ellipseOnMouseDown);
    canvas.on('mouse:move', ellipseOnMouseMove);
    canvas.on('mouse:up', ellipseOnMouseUp);
    $('.shape-button').removeClass('active')
    $(e).addClass('active')
    showShapesControls(true);
}


// start a fakies line to show where the real polyline will go
const ellipseOnMouseDown = (event) => {
    // lock canvas interaction
    // flag that the line is actually down
    lineIsDown = true;
    // get the current mouse coordinats
    let pointer = canvas.getPointer(event.e);
    // save the origin for a minute
    textorigX = pointer.x;
    textorigY = pointer.y;
    // start the line
    let points = [pointer.x, pointer.y, pointer.x, pointer.y];
    currentLine = new fabric.Ellipse({
        originX: 'center',
        originY: 'center',
        left: textorigX,
        top: textorigY,
        stroke: 'grey',
        strokeWidth: 1,
        fill: '',
        strokeDashArray: [5, 2],
        rx: 20,
        ry: 20
    });
    canvas.add(currentLine).renderAll();
    currentLine.setCoords();
}

// make the end of the fakies linecfollow the mouse
const ellipseOnMouseMove = (event) => {
    if (!lineIsDown) return;
    let pointer = canvas.getPointer(event.e);
    let width = Math.abs(pointer.x - textorigX);

    let height = Math.abs(pointer.y - textorigY);
    width = (width < 20) ? 20 : width;
    width = (height < 20) ? 20 : height;
    if (event.e.shiftKey) {
        width = (width > height) ? width : height;
        height = (width > height) ? width : height;
    }
    currentLine.set({
        rx: width,
        ry: height
    });
    canvas.renderAll();
}

// actually make the real line
const ellipseOnMouseUp = (event) => {
    // unlock the canvas for future work
    unlockAll();
    // clean up events
    canvas.off('mouse:down', ellipseOnMouseDown);
    canvas.off('mouse:move', ellipseOnMouseMove);
    canvas.off('mouse:up', ellipseOnMouseUp);
    // clean up cursor
    canvas.defaultCursor = 'auto';
    lineisDown = false;
    // get the final coordinates

// make a polyline

    let ellipse = new fabric.Ellipse({
        originX: 'center',
        originY: 'center',
        stroke: 'black',
        fill: '',
        rx: currentLine.rx,
        ry: currentLine.ry,
        top: currentLine.top,
        left: currentLine.left,
        strokeUniform: true,
    });
    // remove fakies guideline
    canvas.remove(currentLine).renderAll();
    // add the polyline
    canvas.add(ellipse);
    canvas.setActiveObject(ellipse);
    canvas.renderAll();
    // enter editing mode- avoids a bug but also looks cool
    // polyline.enterEditingMode();

    // clean up the active look on the button
    $('.shape-button').removeClass('active')
}


// const addPolygon = (sides, top, left, title = "editable shape") => {
//     console.log('adding polygon')
//     let points = [];
//     if (sides < 3) {
//         points = [{x: 50, y: 50}, {x: 150, y: 50}];
//     } else {
//         points = PolyGen(sides, 50, top, left)
//     }
//     let obj = new fabric.Polygon(points, {
//         strokeWidth: 1,
//         objectCaching: false,
//         stroke: 'black',
//         // fill: 'rgba(42,168,204,0.2)',
//         strokeDashArray: ["0", "0"]
//     });
//     obj.left = left / canvas.getZoom() + (obj.getScaledWidth() / 2);
//     obj.top = top / canvas.getZoom() + (obj.getScaledHeight() / 2);
//     obj.set({originX: 'center', originY: 'center'});
//     obj.set('title', title)
//     canvas.setActiveObject(obj);
//     canvas.add(obj).requestRenderAll();
// }

// add a clipart image from an ID
// const addShape = (id, top, left) => {
//     console.log('adding shape')
//     wait(true);
//
//
//     wait(false)
// }


//Custom formatting function for font picker select2 component
const formatBorderStyleResults = (item) => {
    let $label = $(
        '<span >' + item.svg + item.text + '</span>'
    );
    return $label;
};


function fill_color_remove() {
    $('#update_shape_fill_colour').colorpicker('setValue', '#ffffff');//.trigger('change');
}


function updateShapeBackgroundColor(event) {
    // console.log(event)
    //  event.stopPropagation();
    //  event.stopImmediatePropagation();
    //  event.preventDefault()
    let currentElement = canvas.getActiveObject();
    currentElement.set('fill', $("#update_shape_fill_colour").colorpicker('getValue'));
    canvas.requestRenderAll();
}


const showShapesControls = (islines = false) => {

    $("#colourway_btn").collapse('hide');
    $("#shapes_tools_dd").show();
    $("#update_selection_btn").hide();
    $("#show_scale_btn").hide();
    $("#align_btn").hide();

    $("#opacityRange").val(100);
    $("#update_shape_border_weight").val(1);
    if (islines) {
        $("#no_border_control").hide();
        $("#update_shape_fill_colour_div").hide();
    }
    $("#border_weight_range").val(1);
    $("#border_weight_number").val(1)

    $('.top-buttons_right').css('display', "");
    $(".image_context_menu").css('display', "flex");
    $('#groupImage,#alignItems').css("display", "none");
    $('#colorWays, #scaleObject, #groupObjectButton,#ungroupObjectButton').css("display", "none");
    $("#editShapes").css("display", "");
}

const hideShapesControls = () => {
    $("#update_shape_fill_colour").show();
    $("#no_border_control").show();
    $("#shapes_tools_dd").hide();
}

///////////////////////////////////////////////////////////////////////////////////////
//
//Shapes handles
//
///////////////////////////////////////////////////////////////////////////////////////
//
// // define a function that can locate the controls.
// // this function will be used both for drawing and for interaction.
// function polygonPositionHandler(dim, finalMatrix, fabricObject) {
//     var x = (fabricObject.points[this.pointIndex].x - fabricObject.pathOffset.x),
//         y = (fabricObject.points[this.pointIndex].y - fabricObject.pathOffset.y);
//     return fabric.util.transformPoint(
//         {x: x, y: y},
//         fabric.util.multiplyTransformMatrices(
//             fabricObject.canvas.viewportTransform,
//             fabricObject.calcTransformMatrix()
//         )
//     );
// }
//
// // define a function that will define what the control does
// // this function will be called on every mouse move after a control has been
// // clicked and is being dragged.
// // The function receive as argument the mouse event, the current trasnform object
// // and the current position in canvas coordinate
// // transform.target is a reference to the current object being transformed,
// const actionHandler = (eventData, transform, x, y) => {
//     let polygon = transform.target,
//         currentControl = polygon.controls[polygon.__corner],
//         mouseLocalPosition = polygon.toLocalPoint(new fabric.Point(x, y), 'center', 'center'),
//         polygonBaseSize = polygon._getNonTransformedDimensions(),
//         size = polygon._getTransformedDimensions(0, 0),
//         finalPointPosition = {
//             x: mouseLocalPosition.x * polygonBaseSize.x / size.x + polygon.pathOffset.x,
//             y: mouseLocalPosition.y * polygonBaseSize.y / size.y + polygon.pathOffset.y
//         };
//     polygon.points[currentControl.pointIndex] = finalPointPosition;
//     return true;
// }
//
// // define a function that can keep the polygon in the same position when we change its
// // width/height/top/left.
// const anchorWrapper = (anchorIndex, fn) => {
//     return function (eventData, transform, x, y) {
//         let fabricObject = transform.target,
//             absolutePoint = fabric.util.transformPoint({
//                 x: (fabricObject.points[anchorIndex].x - fabricObject.pathOffset.x),
//                 y: (fabricObject.points[anchorIndex].y - fabricObject.pathOffset.y),
//             }, fabricObject.calcTransformMatrix()),
//             actionPerformed = fn(eventData, transform, x, y),
//             newDim = fabricObject._setPositionDimensions({}),
//             polygonBaseSize = fabricObject._getNonTransformedDimensions(),
//             newX = (fabricObject.points[anchorIndex].x - fabricObject.pathOffset.x) / polygonBaseSize.x,
//             newY = (fabricObject.points[anchorIndex].y - fabricObject.pathOffset.y) / polygonBaseSize.y;
//         fabricObject.setPositionByOrigin(absolutePoint, newX + 0.5, newY + 0.5);
//         return actionPerformed;
//     }
// }
//
// // turn on the polygon points editing
// const polygonEditing = (poly) => {
//     // clone what are you copying since you
//     // may want copy and paste on different moment.
//     // and you do not want the changes happened
//     // later to reflect on the copy.
//     // let poly = canvas.getActiveObject();
//     if (poly.objectCaching) {
//         poly.objectCaching = false;
//     }
//
//     var lastControl = poly.points.length - 1;
//     poly.cornerStyle = 'circle';
//     poly.cornerColor = 'rgba(0,0,255,0.5)';
//     poly.controls = poly.points.reduce(function (acc, point, index) {
//         acc['p' + index] = new fabric.Control({
//             positionHandler: polygonPositionHandler,
//             actionHandler: anchorWrapper(index > 0 ? index - 1 : lastControl, actionHandler),
//             actionName: 'modifyPolygon',
//             pointIndex: index
//         });
//         return acc;
//     }, {});
//     canvas.requestRenderAll();
//
// }
//
// const polygonEditingOff = (poly) => {
//     poly.cornerColor = 'rgb(178,204,255)';
//     poly.cornerStyle = 'rect';
//     poly.controls = fabric.Object.prototype.controls;
//     canvas.requestRenderAll();
// }
//
//
// const polygonPoints = (x, y, radius, sides) => {
//     let points = [{x: x, y: y}];
//     if (sides < 3) {
//         return;
//     } else {
//         let a = ((Math.PI * 2) / sides);
//         for (let i = 0; i < sides; i++) {
//             points.push({x: radius * Math.cos(a * i), y: Math.sin(a * i)})
//         }
//         console.log('points are:');
//         console.log(points);
//         return points;
//     }
// }

const PolyGen = (count, radius, orig_x, orig_y) => {
    count = count || 3; // sides
    let points = [];
    if (!radius) {
        radius = 50;
    }// radius
    for (i = 0; i < count; i++) {
        points.push({
            x: orig_x + radius * Math.sin(i * (2 * Math.PI / count) - (Math.PI / count)),
            y: orig_y + radius * Math.cos(i * (2 * Math.PI / count) - (Math.PI / count))
        });
    }
    //  console.log(points)
    return points;
}
